@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -115,6 +115,9 @@ extern int	CONFIG_IPMIMANAGER_FORKS;
 extern int	CONFIG_ALERTMANAGER_FORKS;
 extern int	CONFIG_PREPROCMAN_FORKS;
 extern int	CONFIG_PREPROCESSOR_FORKS;
+extern int	CONFIG_LLDMANAGER_FORKS;
+extern int	CONFIG_LLDWORKER_FORKS;
+extern int	CONFIG_ALERTDB_FORKS;
 
 extern unsigned char	process_type;
 extern int		process_num;
@@ -125,7 +128,7 @@ extern int		process_num;
  *                                                                            *
  * Purpose: Returns number of processes depending on process type             *
  *                                                                            *
- * Parameters: process_type - [IN] process type; ZBX_PROCESS_TYPE_*           *
+ * Parameters: proc_type - [IN] process type; ZBX_PROCESS_TYPE_*              *
  *                                                                            *
  * Return value: number of processes                                          *
  *                                                                            *
@@ -192,6 +195,12 @@ int	get_process_type_forks(unsigned char proc_type)
 			return CONFIG_PREPROCMAN_FORKS;
 		case ZBX_PROCESS_TYPE_PREPROCESSOR:
 			return CONFIG_PREPROCESSOR_FORKS;
+		case ZBX_PROCESS_TYPE_LLDMANAGER:
+			return CONFIG_LLDMANAGER_FORKS;
+		case ZBX_PROCESS_TYPE_LLDWORKER:
+			return CONFIG_LLDWORKER_FORKS;
+		case ZBX_PROCESS_TYPE_ALERTSYNCER:
+			return CONFIG_ALERTDB_FORKS;
 	}
 
 	THIS_SHOULD_NEVER_HAPPEN;
@@ -211,14 +220,12 @@ int	get_process_type_forks(unsigned char proc_type)
  ******************************************************************************/
 int	init_selfmon_collector(char **error)
 {
-	const char	*__function_name = "init_selfmon_collector";
 	size_t		sz, sz_array, sz_process[ZBX_PROCESS_TYPE_COUNT], sz_total;
 	char		*p;
-	struct tms	buf;
 	unsigned char	proc_type;
 	int		proc_num, process_forks, ret = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	sz_total = sz = sizeof(zbx_selfmon_collector_t);
 	sz_total += sz_array = sizeof(zbx_stat_process_t *) * ZBX_PROCESS_TYPE_COUNT;
@@ -226,7 +233,7 @@ int	init_selfmon_collector(char **error)
 	for (proc_type = 0; ZBX_PROCESS_TYPE_COUNT > proc_type; proc_type++)
 		sz_total += sz_process[proc_type] = sizeof(zbx_stat_process_t) * get_process_type_forks(proc_type);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "%s() size:" ZBX_FS_SIZE_T, __function_name, (zbx_fs_size_t)sz_total);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s() size:" ZBX_FS_SIZE_T, __func__, (zbx_fs_size_t)sz_total);
 
 	if (SUCCEED != zbx_mutex_create(&sm_lock, ZBX_MUTEX_SELFMON, error))
 	{
@@ -253,7 +260,7 @@ int	init_selfmon_collector(char **error)
 	collector = (zbx_selfmon_collector_t *)p; p += sz;
 	collector->process = (zbx_stat_process_t **)p; p += sz_array;
 	collector->ticks_per_sec = sysconf(_SC_CLK_TCK);
-	collector->ticks_sync = times(&buf);
+	collector->ticks_sync = 0;
 
 	for (proc_type = 0; ZBX_PROCESS_TYPE_COUNT > proc_type; proc_type++)
 	{
@@ -263,15 +270,13 @@ int	init_selfmon_collector(char **error)
 		process_forks = get_process_type_forks(proc_type);
 		for (proc_num = 0; proc_num < process_forks; proc_num++)
 		{
-			collector->process[proc_type][proc_num].cache.ticks = collector->ticks_sync;
-			collector->process[proc_type][proc_num].cache.state = ZBX_PROCESS_STATE_BUSY;
-			collector->process[proc_type][proc_num].cache.ticks_flush = collector->ticks_sync;
+			collector->process[proc_type][proc_num].cache.state = ZBX_PROCESS_STATE_IDLE;
 		}
 	}
 
 	ret = SUCCEED;
 out:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() collector:%p", __function_name, (void *)collector);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s() collector:%p", __func__, (void *)collector);
 
 	return ret;
 }
@@ -287,9 +292,7 @@ out:
  ******************************************************************************/
 void	free_selfmon_collector(void)
 {
-	const char	*__function_name = "free_selfmon_collector";
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() collector:%p", __function_name, (void *)collector);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() collector:%p", __func__, (void *)collector);
 
 	if (NULL == collector)
 		return;
@@ -308,7 +311,7 @@ void	free_selfmon_collector(void)
 
 	zbx_mutex_destroy(&sm_lock);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
 /******************************************************************************
@@ -331,7 +334,21 @@ void	update_selfmon_counter(unsigned char state)
 		return;
 
 	process = &collector->process[process_type][process_num - 1];
-	ticks = times(&buf);
+
+	if (-1 == (ticks = times(&buf)))
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "cannot get process times: %s", zbx_strerror(errno));
+		process->cache.state = state;
+		return;
+	}
+
+	if (0 == process->cache.ticks_flush)
+	{
+		process->cache.ticks_flush = ticks;
+		process->cache.state = state;
+		process->cache.ticks = ticks;
+		return;
+	}
 
 	/* update process statistics in local cache */
 	process->cache.counter[process->cache.state] += ticks - process->cache.ticks;
@@ -377,14 +394,25 @@ void	update_selfmon_counter(unsigned char state)
  ******************************************************************************/
 void	collect_selfmon_stats(void)
 {
-	const char		*__function_name = "collect_selfmon_stats";
 	zbx_stat_process_t	*process;
 	clock_t			ticks, ticks_done;
 	struct tms		buf;
 	unsigned char		proc_type, i;
 	int			proc_num, process_forks, index, last;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
+
+	if (-1 == (ticks = times(&buf)))
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "cannot get process times: %s", zbx_strerror(errno));
+		goto out;
+	}
+
+	if (0 == collector->ticks_sync)
+	{
+		collector->ticks_sync = ticks;
+		goto out;
+	}
 
 	if (MAX_HISTORY <= (index = collector->first + collector->count))
 		index -= MAX_HISTORY;
@@ -399,7 +427,6 @@ void	collect_selfmon_stats(void)
 
 	LOCK_SM;
 
-	ticks = times(&buf);
 	ticks_done = ticks - collector->ticks_sync;
 
 	for (proc_type = 0; proc_type < ZBX_PROCESS_TYPE_COUNT; proc_type++)
@@ -435,8 +462,8 @@ void	collect_selfmon_stats(void)
 	collector->ticks_sync = ticks;
 
 	UNLOCK_SM;
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
 }
 
 /******************************************************************************
@@ -456,15 +483,14 @@ void	collect_selfmon_stats(void)
  * Author: Alexander Vladishev                                                *
  *                                                                            *
  ******************************************************************************/
-void	get_selfmon_stats(unsigned char proc_type, unsigned char aggr_func, int proc_num,
-		unsigned char state, double *value)
+void	get_selfmon_stats(unsigned char proc_type, unsigned char aggr_func, int proc_num, unsigned char state,
+		double *value)
 {
-	const char	*__function_name = "get_selfmon_stats";
 	unsigned int	total = 0, counter = 0;
 	unsigned char	s;
 	int		process_forks, current;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
 	process_forks = get_process_type_forks(proc_type);
 
@@ -494,14 +520,18 @@ void	get_selfmon_stats(unsigned char proc_type, unsigned char aggr_func, int pro
 	for (; proc_num < process_forks; proc_num++)
 	{
 		zbx_stat_process_t	*process;
-		unsigned short		one_total = 0, one_counter;
+		unsigned int		one_total = 0, one_counter;
 
 		process = &collector->process[proc_type][proc_num];
 
 		for (s = 0; s < ZBX_PROCESS_STATE_COUNT; s++)
-			one_total += process->h_counter[s][current] - process->h_counter[s][collector->first];
+		{
+			one_total += (unsigned short)(process->h_counter[s][current] -
+					process->h_counter[s][collector->first]);
+		}
 
-		one_counter = process->h_counter[state][current] - process->h_counter[state][collector->first];
+		one_counter = (unsigned short)(process->h_counter[state][current] -
+				process->h_counter[state][collector->first]);
 
 		switch (aggr_func)
 		{
@@ -532,7 +562,108 @@ unlock:
 
 	*value = (0 == total ? 0 : 100. * (double)counter / (double)total);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __func__);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_get_all_process_stats                                        *
+ *                                                                            *
+ * Purpose: retrieves internal metrics of all running processes based on      *
+ *          process type                                                      *
+ *                                                                            *
+ * Parameters: stats - [OUT] process metrics                                  *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_get_all_process_stats(zbx_process_info_t *stats)
+{
+	int		current, ret = FAIL;
+	unsigned char	proc_type;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
+
+	LOCK_SM;
+
+	if (1 >= collector->count)
+		goto unlock;
+
+	if (MAX_HISTORY <= (current = (collector->first + collector->count - 1)))
+		current -= MAX_HISTORY;
+
+	for (proc_type = 0; proc_type < ZBX_PROCESS_TYPE_COUNT; proc_type++)
+	{
+		int		proc_num;
+		unsigned int	total_avg = 0, counter_avg_busy = 0, counter_avg_idle = 0,
+				total_max = 0, counter_max_busy = 0, counter_max_idle = 0,
+				total_min = 0, counter_min_busy = 0, counter_min_idle = 0;
+
+		stats[proc_type].count = get_process_type_forks(proc_type);
+
+		for (proc_num = 0; proc_num < stats[proc_type].count; proc_num++)
+		{
+			zbx_stat_process_t	*process;
+			unsigned int		one_total = 0, busy_counter, idle_counter;
+			unsigned char		s;
+
+			process = &collector->process[proc_type][proc_num];
+
+			for (s = 0; s < ZBX_PROCESS_STATE_COUNT; s++)
+			{
+				one_total += (unsigned short)(process->h_counter[s][current] -
+						process->h_counter[s][collector->first]);
+			}
+
+			busy_counter = (unsigned short)(process->h_counter[ZBX_PROCESS_STATE_BUSY][current] -
+					process->h_counter[ZBX_PROCESS_STATE_BUSY][collector->first]);
+
+			idle_counter = (unsigned short)(process->h_counter[ZBX_PROCESS_STATE_IDLE][current] -
+					process->h_counter[ZBX_PROCESS_STATE_IDLE][collector->first]);
+
+			total_avg += one_total;
+			counter_avg_busy += busy_counter;
+			counter_avg_idle += idle_counter;
+
+			if (0 == proc_num || busy_counter > counter_max_busy)
+			{
+				counter_max_busy = busy_counter;
+				total_max = one_total;
+			}
+
+			if (0 == proc_num || idle_counter > counter_max_idle)
+			{
+				counter_max_idle = idle_counter;
+				total_max = one_total;
+			}
+
+			if (0 == proc_num || busy_counter < counter_min_busy)
+			{
+				counter_min_busy = busy_counter;
+				total_min = one_total;
+			}
+
+			if (0 == proc_num || idle_counter < counter_min_idle)
+			{
+				counter_min_idle = idle_counter;
+				total_min = one_total;
+			}
+		}
+
+		stats[proc_type].busy_avg = (0 == total_avg ? 0 : 100. * (double)counter_avg_busy / (double)total_avg);
+		stats[proc_type].busy_max = (0 == total_max ? 0 : 100. * (double)counter_max_busy / (double)total_max);
+		stats[proc_type].busy_min = (0 == total_min ? 0 : 100. * (double)counter_min_busy / (double)total_min);
+
+		stats[proc_type].idle_avg = (0 == total_avg ? 0 : 100. * (double)counter_avg_idle / (double)total_avg);
+		stats[proc_type].idle_max = (0 == total_max ? 0 : 100. * (double)counter_max_idle / (double)total_max);
+		stats[proc_type].idle_min = (0 == total_min ? 0 : 100. * (double)counter_min_idle / (double)total_min);
+	}
+
+	ret = SUCCEED;
+unlock:
+	UNLOCK_SM;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_result_string(ret));
+
+	return ret;
 }
 
 static int	sleep_remains;
