@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,28 +22,16 @@
 
 int	SYSTEM_UPTIME(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	kstat_ctl_t	*kc;
-	kstat_t		*kp;
-	kstat_named_t	*kn;
-	time_t		now;
-	int		ret = SYSINFO_RET_FAIL;
-
-	if (NULL == (kc = kstat_open()))
-		return ret;
-
-	if (NULL != (kp = kstat_lookup(kc, "unix", 0, "system_misc")))
+	if (SYSINFO_RET_OK == SYSTEM_BOOTTIME(request, result))
 	{
-		if (-1 != kstat_read(kc, kp, 0))
-		{
-			if (NULL != (kn = (kstat_named_t*)kstat_data_lookup(kp, "boot_time")))
-			{
-				time(&now);
-				SET_UI64_RESULT(result, now - get_kstat_numeric_value(kn));
-				ret = SYSINFO_RET_OK;
-			}
-		}
-	}
-	kstat_close(kc);
+		time_t	now;
 
-	return ret;
+		time(&now);
+
+		result->ui64 = now - result->ui64;
+
+		return SYSINFO_RET_OK;
+	}
+
+	return SYSINFO_RET_FAIL;
 }
